import express from 'express'
import { SearchResultQuestion } from '../utils/classes'
import type { Database } from 'better-sqlite3'
import type { Socket as SocketIoSocket } from 'socket.io'
import http from 'http'
import https from 'https'

export interface QuestionData {
  type: string
  date?: Date | number
  images?: Array<string>
  hashedImages?: Array<string>
  possibleAnswers?: Array<{
    text: string
    selectedByUser: boolean
  }>
}

export interface Question {
  Q: string
  A: string
  data: QuestionData
  cache?: {
    Q: Array<string>
    A: Array<string>
  }
}

export interface Subject {
  Name: string
  Questions: Array<Question>
}

export interface DataFile {
  path: string
  name: string
  locked?: Boolean
  shouldSearch:
    | string
    | {
        location?: {
          val: string
        }
      }
  shouldSave: {
    location?: {
      val: string
    }
    version?: {
      compare: string
      val: string
    }
  }
}

export interface QuestionDb extends DataFile {
  data: Array<Subject>
  index: number
}

export interface User {
  id: number
  pw: string
  created: Date
}

export interface User {
  id: number
  pw: string
  pwRequestCount: number
  avaiblePWRequests: number
  loginCount: number
  createdBy: number
}

export interface Request<T = any> extends express.Request {
  body: T
  cookies: any
  session: any
  busboy: any
  files: any
}

export interface SubmoduleData {
  app: express.Application
  url: string
  publicdirs: Array<string>
  userDB?: Database
  nextdir?: string
  moduleSpecificData?: any
  httpServer: http.Server
  httpsServer: https.Server
}

export interface QuestionFromScript {
  questions: Array<Question>
  testUrl: string
  subj: string
}

export interface DbSearchResult {
  message?: string
  recievedData?: string
  question: Question
  result: SearchResultQuestion[]
  success: boolean
}

export interface RegisteredUserEntry {
  cid: string
  version: string
  installSource: string
  date: string
  userAgent: string
  uid: number
  loginDate: string
}

export interface Submodule {
  dailyAction?: () => void
  load?: () => void
}

export interface ModuleType {
  app: express.Application
  dailyAction?: Function
}

export interface Socket extends SocketIoSocket {
  user: User
}
